import { Injectable, NgZone } from '@angular/core';
import { ShortcutHandler, register, unregister } from '@tauri-apps/api/globalShortcut';
import { EMPTY, from } from 'rxjs';

@Injectable({
  providedIn: 'root'
})
export class ShortcutService {
  bound: Map<ShortcutHandler, string> = new Map<ShortcutHandler, string>();

  constructor(private zone: NgZone) {
  }
  register(shortcut: string, handler: ShortcutHandler) {
    this.bound.set(handler, shortcut);

    return from(register(shortcut, (s) => {
      this.zone.run(() => handler(s));
    }));
  }

  unregister(handler: ShortcutHandler) {
    const shortcut = this.bound.get(handler);
    this.bound.delete(handler);

    return shortcut ? from(unregister(shortcut)) : EMPTY;
  }

  rebind(shortcut: string, handler: ShortcutHandler) {
    const prevShortcut = this.bound.get(handler);
    this.register(shortcut, handler).subscribe(
      {
        error: (_err) => {
          if (prevShortcut) {
            this.register(prevShortcut, handler);
          }
          return EMPTY;
        }
      });
  }

  rebind_from_to(previousShortcut: string, nextShortcut: string) {
    const oldHandler = [...this.bound.entries()].find((entry: [ShortcutHandler, string]) => entry[1] === previousShortcut)?.[0];
    this.rebind(nextShortcut, oldHandler!);
  }
}
